/*
Zernike - algorithme de calcul de polynmes de Zernike

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

public class Zernike {

  int dimension;
  double mask;
  double teta;

// Le constructeur
  public Zernike (int ordre, int _dimension) {
  
    dimension = _dimension;
    double z[][] = new double [dimension][dimension];
  
  }
  
/** Calcule le Zernike normalise (somme(z^2) = 1) de rang zn.
    Convention ici : Zernike(1) = piston, Zernike(2) = tilt, etc...
*/
  public static double[][] calcZernike (int zn, int dimension) {

    double z[][] = new double[dimension][dimension];
    double tempo;
    double r;
    int znm[] = zernumero(zn);
    int n = znm[0];
    int m = znm[1];

    if ( (zn % 2) == 1 ) {  // zn impair
      for ( int i=0 ; i <= (n-m)/2 ; i++ ) {
        tempo = Math.pow(-1.,i)*Math.exp(BibliMath.alngam(n-i+1)-(BibliMath.alngam(i+1)+BibliMath.alngam((n+m)/2.-i+1.)+BibliMath.alngam((n-m)/2.-i+1.)));
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            r = Math.sqrt((j-dimension/2.)*(j-dimension/2.)+(k-dimension/2.)*(k-dimension/2.))/dimension*4.;
            if ( r <= 1. ) {
              z[j][k]=z[j][k]+Math.pow(r,(n-2.*i))*tempo;
            }
            else {
              z[j][k] = 0.;    // en dehors d'un disque de rayon unit (pupille) le Zernike est nul
            }
          }
        }
      }  
      if ( m == 0 ) {
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            z[j][k]=z[j][k]*Math.sqrt(n+1.); 
          }
        }
      }
      else {
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            z[j][k]=z[j][k]*Math.sqrt(2.*(n+1.))*Math.sin(m*Math.atan2((k-dimension/2.),(j-dimension/2.)));
          }
        }
      }
    }
    else {   // zn pair
      for ( int i=0 ; i <= (n-m)/2 ; i++ ) {
        tempo = Math.pow(-1.,i)*Math.exp(BibliMath.alngam(n-i+1)-(BibliMath.alngam(i+1)+BibliMath.alngam((n+m)/2.-i+1.)+BibliMath.alngam((n-m)/2.-i+1.)));
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            r = Math.sqrt((j-dimension/2.)*(j-dimension/2.)+(k-dimension/2.)*(k-dimension/2.))/dimension*4.;
            if ( r <= 1. ) {
              z[j][k]=z[j][k]+Math.pow(r,(n-2.*i))*tempo;
            }
            else {
              z[j][k] = 0.;    // en dehors d'un disque de rayon unit (pupille) le Zernike est nul
            }
          }
        }
      }
      if ( m == 0 ) {
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            z[j][k]=z[j][k]*Math.sqrt(n+1.); 
          }
        }
      }
      else {
        for ( int j=0 ; j < dimension ; j++ ) {
          for ( int k=0 ; k < dimension ; k++ ) {
            z[j][k]=z[j][k]*Math.sqrt(2.*(n+1.))*Math.cos(m*Math.atan2((k-dimension/2.),(j-dimension/2.)));
          }
        }
      }
    }

  return z;
  
  }
  
/** Calcul de l'ordre radial et azimutal d'un Zernike donn.
    Retourne un vecteur  deux valeurs : ordre radial, puis azimutal
*/
  public static int[] zernumero(int zn) {

    int j = 0;
    int resu[] = { 0 , 0 };
    
    for ( int n=0 ; n <=100 ; n++ ) {    
      for ( int m=0 ; m <= n ; m++ ) {      
	if ( (n-m) % 2 == 0 ) { // n-m pair
          j=j+1;
          if ( j == zn ) {
            resu[0] = n;
            resu[1] = m;
            break;
          }
          if ( m != 0 ) {
            j=j+1;
            if ( j == zn ) {
              resu[0] = n;
              resu[1] = m;
              break;
            }
          }
        } 
      }
    }
    return resu;
  }
  
}
